-- Copyright (c) 2011 Cloudera, Inc. All rights reserved.
--
-- Version 21 of the CMF schema. Migration code to be applied on top of the
-- previous version.
--

update SCHEMA_VERSION set VERSION=21;

--
-- Move existing configs from one level to another. The assumption is that
-- these config attributes are globally unique, so it isn't necessary to check
-- that the services in question are actually MapReduce services (which would
-- involve an extra join).

--
-- This config has moved from MR service-level to TT-level.
update CONFIGS
    set ROLE_TYPE='TASKTRACKER' where
        ATTR='mapred_output_compression_type';

--
-- These configs have moved from JT-level to TT-level. The migration is
-- complicated because they may have been set at the JT role type level or at
-- the JT role instance level. We want the JT role instance level config to
-- take precedence if it exists, and the JT role type level config otherwise.

--
-- If a JT role instance level config exists, remove any corresponding JT role
-- type level config; it is uninteresting by definition.
delete from CONFIGS where
    ROLE_TYPE='JOBTRACKER' and
    ATTR in ('mapred_output_compress',
             'mapred_output_compression_codec',
             'mapred_compress_map_output',
             'mapred_child_java_opts_base',
             'mapred_child_java_opts_max_heap',
             'mapred_child_ulimit') and exists
    (select null
     from CONFIGS c2 where
     c2.ROLE_ID is not null and
     CONFIGS.ATTR=c2.ATTR);

--
-- Move any JT role level config to JT role type level. The above statement
-- guarantees that, should this update have any effect, there are no JT role
-- type level configs.
update CONFIGS
    set ROLE_ID=null where
        ROLE_ID is not null and
        ATTR in ('mapred_output_compress',
                 'mapred_output_compression_codec',
                 'mapred_compress_map_output',
                 'mapred_child_java_opts_base',
                 'mapred_child_java_opts_max_heap',
                 'mapred_child_ulimit');

--
-- Change the configs from JT role type level to TT role type level.
update CONFIGS
    set ROLE_TYPE='TASKTRACKER' where
        ATTR in ('mapred_output_compress',
                 'mapred_output_compression_codec',
                 'mapred_compress_map_output',
                 'mapred_child_java_opts_base',
                 'mapred_child_java_opts_max_heap',
                 'mapred_child_ulimit');
